/* ************************************************************************** *
 *      Copyright © IPSL/LSCE, XMLIOServer, Avril 2010 - Octobre 2011         *
 * ************************************************************************** */
/**
 * \file    main_server.cpp
 * \brief   Programme principal du projet.
 * \author  Hervé Ozdoba
 * \version 0.4
 * \date    1er Juin 2011
 *
 * Ce fichier contient le point d'entrée du programme permettant
 * de lancer le serveur d'Entrée/Sortie parallèle.
 */

// XMLIOServer headers
#include "xmlioserver_spl.hpp"

#include "mpi_interface.hpp"
#include "onetcdf4.hpp"
#include "onetcdf4_impl.hpp"

namespace xios = xmlioserver;
//~ using namespace xios::data;
using namespace xios::io;
//~ using namespace xios::tree;
//~ using namespace xios::xml;
using namespace xios::comm;
//~ using namespace xios::func;

// Point d'entrée du programme serveur principal.
int main (int argc, char ** argv, char ** XIOS_UNUSED (env))
{
   try
   {
      CMPIManager::Initialise(&argc, &argv);
      {
         MPI_Comm cworld = CMPIManager::GetCommWorld();
         int rank = CMPIManager::GetCommRank();
         int size = CMPIManager::GetCommSize();
         
         CONetCDF4 output("wk/data/herve.nc", false, &cworld);
         
         CONetCDF4::CNetCDF4Path path;
         path.push_back("ozdoba");
         
         output.addGroup("ozdoba");
         output.setCurrentPath(path);
         output.addDimension("time");
         output.addDimension("x", 10);
         output.addDimension("y", 10);
         
         std::vector<std::string> dims;
         dims.push_back("time");
         dims.push_back("x");
         dims.push_back("y");
         
         std::string toto("toto");
         output.addVariable(toto, NC_INT, dims);

         output.writeAttribute("att1", "llo");
         
         std::vector<int> vvv;
         vvv.push_back(1);
         output.writeAttribute("att1", vvv, &toto);
         vvv.push_back(5);
         output.writeAttribute("att5", vvv, &toto);
         
         boost::multi_array<int, 2> data(boost::extents[2][10]);
         boost::multi_array<int, 2>::iterator it = data.begin(), end = data.end();
         
         for (std::size_t i = 0; i < data.num_elements(); i++) data.data()[i] = rank;

         std::vector<std::size_t> _start;
         std::vector<std::size_t> _count;
         _start.push_back(10*rank/size);
         _count.push_back(2);
         _start.push_back(0);
         _count.push_back(10);
         
         output.writeData( data, toto, 0, false, &_start, &_count);
      }

      CMPIManager::Finalize();
   }
   catch (xios::CException & _exception)
   {
      std::cerr << _exception.getMessage() << std::endl;
      CMPIManager::Finalize();
      return (-1);
   }
   return (0);
}


